# Generated by Django 4.2.20 on 2026-01-06 02:47

# Django
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("georgeforge", "0025_alter_estimated_delivery_date_to_string"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="order",
            name="on_behalf_of",
        ),
    ]



# Original license
# Copyright (c) 2022 Torchbox Ltd and individual contributors.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification,
# are permitted provided that the following conditions are met:
#
#     1. Redistributions of source code must retain the above copyright notice,
#        this list of conditions and the following disclaimer.
#
#     2. Redistributions in binary form must reproduce the above copyright
#        notice, this list of conditions and the following disclaimer in the
#        documentation and/or other materials provided with the distribution.
#
#     3. Neither the name of Torchbox nor the names of its contributors may be used
#        to endorse or promote products derived from this software without
#        specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
# ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
# ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

# Django
from django import forms


class Options:
    """
    An object that serves as a container for configuration options. When a class is defined using
    OptionCollectingMetaclass as its metaclass, any attributes defined on an inner `class Meta`
    will be copied to an Options instance which will then be accessible as the class attribute
    `_meta`.

    The base Options class has no functionality of its own, but exists so that specific
    configuration options can be defined as mixins and collectively merged in to either Options or
    another base class with the same interface such as django.forms.models.ModelFormOptions,
    to arrive at a final class that recognises the desired set of options.
    """

    def __init__(self, options=None):
        pass


class OptionCollectingMetaclass(type):
    """
    Metaclass that handles inner `class Meta` definitions. When a class using
    OptionCollectingMetaclass defines an inner Meta class and an `options_class` attribute
    specifying an Options class, an Options object will be created from it and set as the class
    attribute `_meta`.
    """

    options_class = None

    def __new__(mcs, name, bases, attrs):
        new_class = super().__new__(mcs, name, bases, attrs)
        if mcs.options_class:
            new_class._meta = mcs.options_class(getattr(new_class, "Meta", None))
        return new_class


class PermissionedFormOptionsMixin:
    """Handles the field_permissions option for PermissionedForm"""

    def __init__(self, options=None):
        super().__init__(options)
        self.field_permissions = getattr(options, "field_permissions", None)


class PermissionedFormOptions(PermissionedFormOptionsMixin, Options):
    """Options class for PermissionedForm"""


FormMetaclass = type(forms.Form)


class PermissionedFormMetaclass(OptionCollectingMetaclass, FormMetaclass):
    """
    Extends the django.forms.Form metaclass with support for an inner `class Meta` that accepts
    a `field_permissions` configuration option
    """

    options_class = PermissionedFormOptions


class PermissionedForm(forms.Form, metaclass=PermissionedFormMetaclass):
    """
    An extension to `django.forms.Form` to accept an optional `for_user` keyword argument
    indicating the user the form will be presented to.

    Any fields named in the `field_permissions` dict in Meta will apply a permission test on the
    named permission using `User.has_perm`; if the user lacks that permission, the field will be
    omitted from the form.
    """

    def __init__(self, *args, for_user=None, **kwargs):
        super().__init__(*args, **kwargs)
        if for_user:
            field_perms = self._meta.field_permissions or {}
            for field_name, perm in field_perms.items():
                if not for_user.has_perm(perm):
                    del self.fields[field_name]


class PermissionedModelFormOptions(
    PermissionedFormOptionsMixin, forms.models.ModelFormOptions
):
    """
    Options class for PermissionedModelForm; extends ModelForm's options to accept
    `field_permissions`
    """


class PermissionedModelFormMetaclass(
    PermissionedFormMetaclass, forms.models.ModelFormMetaclass
):
    """
    Metaclass for PermissionedModelForm; extends the ModelForm metaclass to use
    PermissionedModelFormOptions in place of ModelFormOptions and thus accept the
    `field_permissions` option.

    Note that because ModelForm does not participate in the OptionCollectingMetaclass logic, this
    has the slightly hacky effect of letting ModelFormMetaclass construct a ModelFormOptions object
    for the lifetime of ModelFormMetaclass.__new__, which we promptly throw out and recreate as a
    PermissionedModelFormOptions object.
    """

    options_class = PermissionedModelFormOptions


class PermissionedModelForm(
    PermissionedForm, forms.ModelForm, metaclass=PermissionedModelFormMetaclass
):
    """A ModelForm that implements the `for_user` keyword argument from PermissionedForm"""



"""
App Models
"""

from georgeforge import app_settings

# Django
from django.contrib.auth.models import User
from django.core.validators import MinValueValidator
from django.db import models
from django.utils.translation import gettext_lazy as _

# Alliance Auth (External Libs)
from eveuniverse.models import EveSolarSystem, EveType
from invoices.models import Invoice

class General(models.Model):
    """Meta model for app permissions"""

    class Meta:
        """Meta definitions"""

        managed = False
        default_permissions = ()
        permissions = (
            ("place_order", "Can place an order"),
            ("manage_store", "Can manage the store"),
        )


class ForSale(models.Model):
    """An item for sale"""

    class Meta:
        default_permissions = ()

    eve_type = models.ForeignKey(
        EveType,
        verbose_name=_("EVE Type"),
        on_delete=models.CASCADE,
        limit_choices_to={"published": 1},
    )

    description = models.TextField(
        _("Description"),
        blank=True,
        max_length=4096,
    )

    price = models.DecimalField(
        _("Price"),
        max_digits=15,
        decimal_places=2,
        help_text=_("Cost per unit"),
        validators=[MinValueValidator(1)],
    )

    deposit = models.DecimalField(
        _("Deposit"),
        default=0,
        max_digits=15,
        decimal_places=2,
        help_text=_("Deposit per unit"),
    )


class DeliverySystem(models.Model):
    """A SolarSystem Available for orders to be delivered to"""

    class Meta:
        default_permissions = ()

    system = models.ForeignKey(
        EveSolarSystem,
        verbose_name=_("Solar System"),
        on_delete=models.CASCADE,
    )

    friendly_name = models.TextField(
        _("Friendly Name"),
        max_length=32,
        default=None,
        null=True,
        blank=True,
    )

    enabled = models.BooleanField(
        default=True,
    )

    @property
    def friendly(self):
        if self.friendly_name:
            return f"{self.system.name} - {self.friendly_name}"
        return self.system.name


class Order(models.Model):
    """An order from a user"""

    class Meta:
        default_permissions = ()

    class OrderStatus(models.IntegerChoices):
        """ """

        PENDING = 10, _("Pending")
        AWAITING_DEPOSIT = 20, _("Awaiting Deposit")
        DEPOSIT_RECIEVED = 25, _("Deposit Recieved")
        BUILDING_PARTS = 30, _("Building Parts")
        BUILDING_HULL = 35, _("Building Hull")
        AWAITING_FINAL_PAYMENT = 40, _("Contract Up")
        DELIVERED = 50, _("Delivered")
        REJECTED = 60, _("Rejected")

    user = models.ForeignKey(
        User,
        verbose_name=_("Purchaser"),
        on_delete=models.RESTRICT,
    )

    price = models.DecimalField(
        _("Price per unit"),
        max_digits=15,
        decimal_places=2,
        help_text=_("Cost per unit"),
    )

    totalcost = models.DecimalField(
        _("Total Order cost"),
        max_digits=25,
        decimal_places=2,
        help_text=_("Total Order cost"),
    )

    deposit = models.DecimalField(
        _("Deposit required per unit"),
        max_digits=25,
        decimal_places=2,
        help_text=_("Deposit required per unit"),
    )

    paid = models.DecimalField(
        _("Amount Paid"),
        default=0,
        max_digits=25,
        decimal_places=2,
        help_text=_("Amount paid"),
    )

    eve_type = models.ForeignKey(
        EveType,
        verbose_name=_("EVE Type"),
        on_delete=models.CASCADE,
        limit_choices_to={"published": 1},
    )

    quantity = models.PositiveIntegerField(
        _("Quantity"),
        default=1,
        validators=[MinValueValidator(1)],
    )

    notes = models.TextField(
        _("Notes"),
        blank=True,
        max_length=4096,
    )

    description = models.TextField(
        _("Description"),
        blank=True,
        max_length=4096,
    )

    deliverysystem = models.ForeignKey(
        EveSolarSystem,
        verbose_name=_("Delivery System"),
        on_delete=models.CASCADE,
    )

    status = models.IntegerField(_("Status"), choices=OrderStatus.choices)

    cart_session_id = models.CharField(
        _("Cart Session ID"),
        max_length=64,
        db_index=True,
    )

    estimated_delivery_date = models.CharField(
        _("Estimated Delivery Date"),
        max_length=50,
        blank=True,
        help_text=_("Estimated date when the order will be delivered"),
    )

    @classmethod
    def ping_invoice(cls, inv):
        # ping the invoice to the user ( if we know them )
        message = f"{inv.note}\n\nPlease check auth for how to pay!\n"
        inv.notify(message, title=app_settings.GEORGEFORGE_APP_NAME)

    @classmethod
    def generate_invoice(cls, cid, oid, price, due_date):
        msg = f"Deposit invoice for Order #{oid}"
        ref = f"GF-DEP-{str(oid)}"
        return Invoice.objects.create(character_id=cid,
                                      amount=round(price, -3),
                                      invoice_ref=ref,
                                      note=msg,
                                      due_date=due_date)


"""App URLs"""

# Django
from django.urls import path

# George Forge
from georgeforge import views

app_name: str = "georgeforge"

urlpatterns = [
    path("store", views.store, name="store"),
    path("orders", views.my_orders, name="my_orders"),
    path("orders/all", views.all_orders, name="all_orders"),
    path("api/cart/checkout", views.cart_checkout_api, name="cart_checkout_api"),
    path(
        "api/orders/<int:order_id>/status",
        views.order_update_status,
        name="order_update_status",
    ),
    path(
        "api/orders/<int:order_id>/paid",
        views.order_update_paid,
        name="order_update_paid",
    ),
    path(
        "api/orders/<int:order_id>/quantity",
        views.order_update_quantity,
        name="order_update_quantity",
    ),
    path(
        "api/orders/<int:order_id>/system",
        views.order_update_system,
        name="order_update_system",
    ),
    path(
        "api/orders/<int:order_id>/estimated-date",
        views.order_update_estimated_date,
        name="order_update_estimated_date",
    ),
    path("bulk_import_form", views.bulk_import_form, name="bulk_import_form"),
    path("bulk_import_form/export", views.export_offers, name="export_offers"),
    path("admin/tasks", views.admin_create_tasks, name="admin_create_tasks"),
]



# Django
from django import forms
from django.utils.translation import gettext_lazy as _


class BulkImportStoreItemsForm(forms.Form):
    """ """

    data = forms.CharField(
        label=_("CSV Paste"),
        empty_value=_("Item Name,Description,Price,Deposit"),
        widget=forms.Textarea(
            attrs={
                "rows": "15",
                "placeholder": _("Item Name,Description,Price,Deposit"),
            }
        ),
    )



# Generated by Django 4.2.27 on 2026-01-24 20:14

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('georgeforge', '0028_alter_order_estimated_delivery_date'),
    ]

    operations = [
        migrations.AlterField(
            model_name='order',
            name='status',
            field=models.IntegerField(choices=[(10, 'Pending'), (20, 'Awaiting Deposit'), (25, 'Deposit Recieved'), (30, 'Building Parts'), (35, 'Building Hull'), (40, 'Contract Up'), (50, 'Delivered'), (60, 'Rejected')], verbose_name='Status'),
        ),
    ]



# Generated by Django 4.2.27 on 2026-01-24 19:11

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('georgeforge', '0027_alter_order_estimated_delivery_date'),
    ]

    operations = [
        migrations.AlterField(
            model_name='order',
            name='estimated_delivery_date',
            field=models.CharField(blank=True, help_text='Estimated date when the order will be delivered', max_length=50, verbose_name='Estimated Delivery Date'),
        ),
    ]



# Standard Library
import itertools

# Django
from django.core.management import call_command
from django.core.management.base import BaseCommand

# George Forge
from georgeforge import __title__
from georgeforge.app_settings import GEORGEFORGE_CATEGORIES


class Command(BaseCommand):
    """ """

    help = "Preloads data required for this app from ESI"

    def handle(self, *args, **options):
        """

        :param *args:
        :param **options:

        """

        categories: list[list[str]] = [
            ["--category_id", str(x)] for x in GEORGEFORGE_CATEGORIES
        ]

        call_command(
            "eveuniverse_load_types",
            "--noinput",
            __title__,
            *itertools.chain.from_iterable(categories),
        )
        call_command("eveuniverse_load_data", "--noinput", "map")



"""App Configuration"""

# Django
from django.apps import AppConfig

# George Forge
from georgeforge import __version__


class GeorgeForgeConfig(AppConfig):
    """App Config"""

    name = "georgeforge"
    label = "georgeforge"
    verbose_name = f"George Forge v{__version__}"



# Generated manually

# Django
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("georgeforge", "0021_alter_order_deliverysystem"),
    ]

    operations = [
        migrations.AddField(
            model_name="order",
            name="cart_session_id",
            field=models.CharField(
                blank=True,
                db_index=True,
                max_length=64,
                null=True,
                unique=True,
                verbose_name="Cart Session ID",
            ),
        ),
    ]



"""App Views"""

# Standard Library
import csv
import itertools
import logging
import uuid
from operator import attrgetter

# Django
from django.contrib import messages
from django.contrib.auth.decorators import login_required, permission_required
from django.core.exceptions import ObjectDoesNotExist, ValidationError
from django.core.handlers.wsgi import WSGIRequest
from django.http import HttpResponse, JsonResponse
from django.shortcuts import redirect, render
from django.template.defaultfilters import pluralize
from django.utils.translation import gettext_lazy as _
from django.views.decorators.http import require_POST
from django_celery_beat.models import CrontabSchedule, PeriodicTask

# Alliance Auth (External Libs)
from eveuniverse.models import EveSolarSystem, EveType

# George Forge
from georgeforge.forms import BulkImportStoreItemsForm
from georgeforge.models import DeliverySystem, ForSale, Order
from georgeforge.tasks import (
    send_deliverydateupdate_dm,
    send_order_webhook,
    send_statusupdate_dm,
    send_order_invoice
)

from . import app_settings

logger = logging.getLogger(__name__)


@login_required
@permission_required("georgeforge.place_order")
def store(request: WSGIRequest) -> HttpResponse:
    """Store view

    :param request: WSGIRequest:

    """

    for_sale = (
        ForSale.objects.select_related("eve_type__eve_group")
        .all()
        .order_by("eve_type__eve_group__name")
    )

    groups = [
        (key, list(l))
        for key, l in itertools.groupby(
            for_sale, key=attrgetter("eve_type.eve_group.name")
        )
    ]
    groups.sort(key=lambda pair: max(entry.price for entry in pair[1]), reverse=True)

    delivery_systems = DeliverySystem.objects.filter(enabled=True).select_related(
        "system"
    )

    context = {
        "for_sale": groups,
        "delivery_systems": delivery_systems,
        "user_id": request.user.id,
    }

    return render(request, "georgeforge/views/store.html", context)


@login_required
@permission_required("georgeforge.place_order")
def my_orders(request: WSGIRequest) -> HttpResponse:
    """My Orders view

    :param request: WSGIRequest:

    """

    my_orders = (
        Order.objects.select_related()
        .filter(user=request.user, status__lt=Order.OrderStatus.DELIVERED)
        .order_by("-id")
    )
    done_orders = (
        Order.objects.select_related()
        .filter(user=request.user, status__gte=Order.OrderStatus.DELIVERED)
        .order_by("-id")
    )

    context = {
        "my_orders": my_orders,
        "done_orders": done_orders,
        "user_id": request.user.id,
    }

    return render(request, "georgeforge/views/my_orders.html", context)


@login_required
@permission_required("georgeforge.place_order")
@require_POST
def cart_checkout_api(request: WSGIRequest) -> JsonResponse:
    """Cart checkout API endpoint

    :param request: WSGIRequest:
    :return: JsonResponse:

    """
    # Standard Library
    import json

    try:
        data = json.loads(request.body)
    except json.JSONDecodeError:
        return JsonResponse({"success": False, "error": "Invalid JSON"}, status=400)

    items = data.get("items", [])
    deliverysystem_id = data.get("deliverysystem_id")
    notes = data.get("notes", "")

    if not items:
        return JsonResponse({"success": False, "error": "No items in cart"}, status=400)

    if not deliverysystem_id:
        return JsonResponse(
            {"success": False, "error": "Delivery system required"}, status=400
        )

    try:
        deliverysystem = EveSolarSystem.objects.get(id=deliverysystem_id)
    except EveSolarSystem.DoesNotExist:
        return JsonResponse(
            {"success": False, "error": "Invalid delivery system"}, status=400
        )

    cart_session_id = str(uuid.uuid4())

    items_dict = {}
    for item in items:
        for_sale_id = item.get("for_sale_id")
        quantity = item.get("quantity", 1)

        if quantity < 1:
            return JsonResponse(
                {"success": False, "error": "Minimum quantity 1"}, status=400
            )

        if for_sale_id in items_dict:
            items_dict[for_sale_id] += quantity
        else:
            items_dict[for_sale_id] = quantity

    orders = []

    for for_sale_id, quantity in items_dict.items():
        try:
            for_sale = ForSale.objects.get(id=for_sale_id)
        except ForSale.DoesNotExist:
            return JsonResponse(
                {"success": False, "error": f"Item {for_sale_id} not found"}, status=400
            )

        order = Order.objects.create(
            user=request.user,
            price=for_sale.price,
            totalcost=(for_sale.price * quantity),
            deposit=(for_sale.deposit * quantity),
            eve_type=for_sale.eve_type,
            notes=notes,
            description=for_sale.description,
            status=Order.OrderStatus.PENDING,
            deliverysystem=deliverysystem,
            quantity=quantity,
            estimated_delivery_date="",
            cart_session_id=cart_session_id,
        )

        orders.append(order)

    for order in orders:
        send_order_webhook.delay(order.pk)
        send_statusupdate_dm(order)

    total_deposit = sum(float(order.deposit) for order in orders)
    deposit_instructions = (
        app_settings.GEORGEFORGE_ORDER_DEPOSIT_INSTRUCTIONS if total_deposit > 0 else None
    )

    if deposit_instructions:
        messages.success(request, _(deposit_instructions))

    return JsonResponse(
        {
            "success": True,
            "orders": [
                {
                    "id": order.id,
                    "eve_type": order.eve_type.name,
                    "quantity": order.quantity,
                    "totalcost": float(order.totalcost),
                    "deposit": float(order.deposit),
                }
                for order in orders
            ],
            "cart_session_id": cart_session_id,
            "deposit_instructions": deposit_instructions,
        }
    )


@login_required
@permission_required("georgeforge.manage_store")
def all_orders(request: WSGIRequest) -> HttpResponse:
    """Order Management handler/view

    :param request: WSGIRequest:

    """
    orders = (
        Order.objects.select_related()
        .filter(status__lt=Order.OrderStatus.DELIVERED)
        .order_by("-id")
    )
    done_orders = (
        Order.objects.select_related()
        .filter(status__gte=Order.OrderStatus.DELIVERED)
        .order_by("-id")
    )
    dsystems = []
    for x in DeliverySystem.objects.select_related().all():
        dsystems.append([x.system.id, x.friendly])
    context = {
        "all_orders": orders,
        "done_orders": done_orders,
        "status": Order.OrderStatus.choices,
        "dsystems": dsystems,
    }

    return render(request, "georgeforge/views/all_orders.html", context)


@login_required
@permission_required("georgeforge.manage_store")
@require_POST
def order_update_status(request: WSGIRequest, order_id: int) -> JsonResponse:
    """AJAX endpoint to update order status

    :param request: WSGIRequest:
    :param order_id: Order ID:
    :return: JsonResponse:

    """
    try:
        order = Order.objects.get(id=order_id)
    except Order.DoesNotExist:
        return JsonResponse({"success": False, "error": "Order not found"}, status=404)

    try:
        status = int(request.POST.get("value"))
    except (ValueError, TypeError):
        return JsonResponse(
            {"success": False, "error": "Invalid status value"}, status=400
        )

    if status not in dict(Order.OrderStatus.choices).keys():
        return JsonResponse(
            {"success": False, "error": "Not a valid status"}, status=400
        )

    old_status = order.status
    order.status = status
    order.save()

    if order.status != old_status:
        send_statusupdate_dm(order)
        if order.status == Order.OrderStatus.AWAITING_DEPOSIT:
            send_order_invoice(order)

    logger.info(
        f"Updated order {order_id} status from {old_status} to {status} by {request.user}"
    )

    return JsonResponse(
        {
            "success": True,
            "pk": order_id,
            "newValue": status,
            "display": order.get_status_display(),
        }
    )


@login_required
@permission_required("georgeforge.manage_store")
@require_POST
def order_update_paid(request: WSGIRequest, order_id: int) -> JsonResponse:
    """AJAX endpoint to update order paid amount

    :param request: WSGIRequest:
    :param order_id: Order ID:
    :return: JsonResponse:

    """
    try:
        order = Order.objects.get(id=order_id)
    except Order.DoesNotExist:
        return JsonResponse({"success": False, "error": "Order not found"}, status=404)

    try:
        paid = float(request.POST.get("value").strip(","))
    except (ValueError, TypeError, AttributeError):
        return JsonResponse(
            {"success": False, "error": "Invalid paid amount"}, status=400
        )

    if paid < 0.00:
        return JsonResponse(
            {"success": False, "error": "Negative payment not allowed"}, status=400
        )

    order.paid = paid
    order.save()

    logger.info(f"Updated order {order_id} paid amount to {paid} by {request.user}")

    return JsonResponse({"success": True, "pk": order_id, "newValue": paid})


@login_required
@permission_required("georgeforge.manage_store")
@require_POST
def order_update_quantity(request: WSGIRequest, order_id: int) -> JsonResponse:
    """AJAX endpoint to update order quantity

    :param request: WSGIRequest:
    :param order_id: Order ID:
    :return: JsonResponse:

    """
    try:
        order = Order.objects.get(id=order_id)
    except Order.DoesNotExist:
        return JsonResponse({"success": False, "error": "Order not found"}, status=404)

    try:
        quantity = int(request.POST.get("value"))
    except (ValueError, TypeError):
        return JsonResponse({"success": False, "error": "Invalid quantity"}, status=400)

    if quantity < 1:
        return JsonResponse(
            {"success": False, "error": "Minimum quantity is 1"}, status=400
        )

    order.quantity = quantity
    order.totalcost = order.price * quantity
    order.save()

    logger.info(f"Updated order {order_id} quantity to {quantity} by {request.user}")

    return JsonResponse({"success": True, "pk": order_id, "newValue": quantity})


@login_required
@permission_required("georgeforge.manage_store")
@require_POST
def order_update_system(request: WSGIRequest, order_id: int) -> JsonResponse:
    """AJAX endpoint to update order delivery system

    :param request: WSGIRequest:
    :param order_id: Order ID:
    :return: JsonResponse:

    """
    try:
        order = Order.objects.get(id=order_id)
    except Order.DoesNotExist:
        return JsonResponse({"success": False, "error": "Order not found"}, status=404)

    try:
        system_id = int(request.POST.get("value"))
    except (ValueError, TypeError):
        return JsonResponse(
            {"success": False, "error": "Invalid system ID"}, status=400
        )

    try:
        deliverysystem = EveSolarSystem.objects.get(id=system_id)
    except EveSolarSystem.DoesNotExist:
        return JsonResponse(
            {"success": False, "error": "Delivery system not found"}, status=404
        )

    order.deliverysystem = deliverysystem
    order.save()

    logger.info(
        f"Updated order {order_id} delivery system to {system_id} by {request.user}"
    )

    return JsonResponse(
        {
            "success": True,
            "pk": order_id,
            "newValue": system_id,
            "display": deliverysystem.name,
        }
    )


@login_required
@permission_required("georgeforge.manage_store")
@require_POST
def order_update_estimated_date(request: WSGIRequest, order_id: int) -> JsonResponse:
    """AJAX endpoint to update order estimated delivery date

    :param request: WSGIRequest:
    :param order_id: Order ID:
    :return: JsonResponse:

    """
    try:
        order = Order.objects.get(id=order_id)
    except Order.DoesNotExist:
        return JsonResponse({"success": False, "error": "Order not found"}, status=404)

    date_value = request.POST.get("value")

    old_date = order.estimated_delivery_date
    if not date_value or date_value.strip() == "":
        order.estimated_delivery_date = ""
    else:
        order.estimated_delivery_date = date_value.strip()

    order.save()

    if order.estimated_delivery_date != old_date:
        send_deliverydateupdate_dm(order)

    logger.info(
        f"Updated order {order_id} estimated delivery date to {order.estimated_delivery_date} by {request.user}"
    )

    return JsonResponse(
        {
            "success": True,
            "pk": order_id,
            "newValue": (
                str(order.estimated_delivery_date)
                if order.estimated_delivery_date
                else ""
            ),
        }
    )


@login_required
@permission_required("georgeforge.manage_store")
def bulk_import_form(request: WSGIRequest) -> HttpResponse:
    """

    :param request: WSGIRequest:

    """
    if request.method == "POST":
        form = BulkImportStoreItemsForm(request.POST)

        if form.is_valid():
            data = form.cleaned_data["data"]
            parsed = [
                row
                for row in csv.DictReader(
                    data.splitlines(),
                    fieldnames=["Item Name", "Description", "Price", "Deposit"],
                )
            ]
            ForSale.objects.all().delete()

            had_error = 0

            for item in parsed:
                try:
                    eve_type = EveType.objects.filter(
                        eve_group__eve_category_id__in=app_settings.GEORGEFORGE_CATEGORIES
                    ).get(name=item["Item Name"])

                    ForSale.objects.create(
                        eve_type=eve_type,
                        description=item["Description"],
                        price=item["Price"],
                        deposit=item["Deposit"],
                    )
                except ObjectDoesNotExist:
                    messages.warning(
                        request,
                        _("%(name)s does not exist and was not added")
                        % {"name": item["Item Name"]},
                    )
                    had_error += 1
                except ValidationError as ex:
                    messages.warning(
                        request,
                        _("%(name)s had a validation error: %(error)s")
                        % {"name": item["Item Name"], "error": ex.message}
                        % ex.params,
                    )
                    had_error += 1

            imported = len(parsed) - had_error

            if imported > 0:
                messages.success(
                    request,
                    _("Imported %(n)s item%(plural)s")
                    % {"n": imported, "plural": pluralize(imported)},
                )

            return redirect("georgeforge:bulk_import_form")

    context = {"form": BulkImportStoreItemsForm()}

    return render(request, "georgeforge/views/bulk_import_form.html", context)


@login_required
@permission_required("georgeforge.manage_store")
def export_offers(request: WSGIRequest) -> HttpResponse:
    """

    :param request: WSGIRequest:

    """
    response = HttpResponse(
        content_type="text/csv",
        headers={"Content-Disposition": 'attachment; filename="auth_forsale.csv"'},
    )

    writer = csv.writer(response)
    for listing in ForSale.objects.all():
        writer.writerow(
            [listing.eve_type.name, listing.description, listing.price, listing.deposit]
        )
    return response

@login_required
@permission_required('georgeforge.manage_store')
def admin_create_tasks(request):
    schedule_invoice_status, _ = CrontabSchedule.objects.get_or_create(minute='15,30,45',
                                                                       hour='*',
                                                                       day_of_week='*',
                                                                       day_of_month='*',
                                                                       month_of_year='*',
                                                                       timezone='UTC'
                                                                       )

    PeriodicTask.objects.update_or_create(
        task='georgeforge.tasks.check_invoice_status',
        defaults={
            'crontab': schedule_invoice_status,
            'name': 'GeorgeForge: Scan deposits',
            'enabled': True
        }
    )
    messages.info(
        request, "Created/Reset Invoice Task to defaults")

    return redirect('georgeforge:store')


# Generated migration for estimated_delivery_date field

# Django
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("georgeforge", "0023_remove_cart_session_id_unique"),
    ]

    operations = [
        migrations.AddField(
            model_name="order",
            name="estimated_delivery_date",
            field=models.DateField(
                blank=True,
                help_text="Estimated date when the order will be delivered",
                null=True,
                verbose_name="Estimated Delivery Date",
            ),
        ),
    ]
